@@ -2,7 +2,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
  Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
@@ -17,7 +17,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -485,8 +485,8 @@ int _slrn_art_wrap_article (Slrn_Article_Type *a) /*{{{*/
    unsigned int wrap_mode = Slrn_Wrap_Mode;
    int wrap_width = Slrn_Wrap_Width;
 
-   if (wrap_width < 5) wrap_width = 5; /* arbitrary */
-   else if (wrap_width > SLtt_Screen_Cols) wrap_width = SLtt_Screen_Cols;
+   if ((wrap_width < 5) || (wrap_width > SLtt_Screen_Cols))
+     wrap_width = SLtt_Screen_Cols;
 
    if (a == NULL)
      return -1;
