@@ -2,4 +2,4 @@
 #error license is more restrictive than the one of slrn itself.
 
 #error If you want to compile slrn for VMS and need this file, please get it
-#error via anonymous FTP from slrn.sourceforge.net in pub/slrn/clientlib.tar.gz
+#error from http://slrn.sourceforge.net/data/clientlib.tar.gz
