@@ -2,7 +2,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
  Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
@@ -17,7 +17,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -222,7 +222,7 @@ static char *Group_Help [] =
 
 static char *Copyright_Notice [] =
 {
-   " Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>",
+   " Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>",
      "",
      " For parts of it:",
      " Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>",
@@ -250,7 +250,7 @@ static char *Copyright_Notice [] =
      "",
      " You should have received a copy of the GNU General Public License along",
      " with this program; if not, write to the Free Software Foundation, Inc.,",
-     " 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.",
+     " 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.",
 #if ! SLRN_USE_SLTCP
      "",
      " Please note that this version of slrn also contains code published under a",
