@@ -2,7 +2,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
  Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
@@ -17,7 +17,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -901,15 +901,26 @@ static char *generic_article_as_string (int use_orig) /*{{{*/
 }
 /*}}}*/
 
-static char *article_as_string (void) /*{{{*/
+static int push_generic_article_as_string (int w)
 {
-   return generic_article_as_string (0);
+   char *s = generic_article_as_string (w);
+   if (s == NULL)
+     {
+	(void) SLang_push_null ();
+	return -1;
+     }
+   return SLang_push_malloced_string (s);   /* frees s */
+}
+
+static void article_as_string (void) /*{{{*/
+{
+   (void) push_generic_article_as_string (0);
 }
 /*}}}*/
 
-static char *raw_article_as_string (void) /*{{{*/
+static void raw_article_as_string (void) /*{{{*/
 {
-   return generic_article_as_string (1);
+   (void) push_generic_article_as_string (1);
 }
 /*}}}*/
 
@@ -985,27 +996,6 @@ static int article_line_down (int *num) /*{{{*/
 
 /*{{{ Article Mode Header Functions */
 
-/*{{{ Header Flag Variables */
-
-static int Interp_Header_Read = HEADER_READ;
-static int Interp_Header_Tagged = HEADER_TAGGED;
-static int Interp_Header_High_Score = HEADER_HIGH_SCORE;
-static int Interp_Header_Low_Score = HEADER_LOW_SCORE;
-static int Interp_Attr_Blink = SLTT_BLINK_MASK;
-static int Interp_Attr_Bold = SLTT_BOLD_MASK;
-static int Interp_Attr_Rev = SLTT_REV_MASK;
-static int Interp_Attr_Uline = SLTT_ULINE_MASK;
-#ifdef HAVE_SETLOCALE
-# ifdef LC_TIME
-static int Interp_Lc_Time = LC_TIME;
-# endif
-# ifdef LC_CTYPE
-static int Interp_Lc_Ctype = LC_CTYPE;
-# endif
-#endif
-
-/*}}}*/
-
 static void uncollapse_threads (void) /*{{{*/
 {
    if (0 == check_article_mode ())
@@ -1091,10 +1081,19 @@ static int header_next_unread (void)
 
 static int get_header_flags (void) /*{{{*/
 {
+   unsigned int mask = HEADER_HARMLESS_FLAGS_MASK;
+   if (SLang_Num_Function_Args == 1)
+     {
+	int i;
+	if (-1 == SLang_pop_integer (&i))
+	  return -1;
+	if (i) mask = 0xFFFFFFFFU;
+     }
+
    if ((-1 == check_article_mode ())
        || (Slrn_Current_Header == NULL))
      return 0;
-   return (int) (Slrn_Current_Header->flags & HEADER_HARMLESS_FLAGS_MASK);
+   return (int) (Slrn_Current_Header->flags & mask);
 }
 
 /*}}}*/
@@ -1242,9 +1241,6 @@ static int re_author_search_backward (char *pat)
 
 /*{{{ Group Functions */
 
-static int Interp_Group_Unsubscribed = GROUP_UNSUBSCRIBED;
-static int Interp_Group_New_Group_Flag = GROUP_NEW_GROUP_FLAG;
-
 static int check_group_mode (void)
 {
    if ((Slrn_Current_Mode == NULL)
@@ -1607,7 +1603,7 @@ static void get_charset_intrin (char *name)
 static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 {
    /* MAKE_INTRINSIC_S("parse_rfc2822", parse_rfc2822_intrin, SLANG_VOID_TYPE), */
-   MAKE_INTRINSIC_0("get_charset", get_charset_intrin, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_1("get_charset", get_charset_intrin, SLANG_VOID_TYPE, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_4("charset_convert_string", charset_convert_string, SLANG_VOID_TYPE, S, S, S, I),
    MAKE_INTRINSIC_I("set_display_state", set_display_state, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("rfc1522_decode_string", rfc1522_decode_string, SLANG_VOID_TYPE),
@@ -1628,7 +1624,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("get_prefix_arg", get_prefix_arg, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("locate_header_by_msgid", locate_header_by_msgid, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("article_as_string", article_as_string, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("article_cline_as_string", article_cline_as_string, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("article_cline_number", article_cline_number, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("article_count_lines", article_count_lines, SLANG_INT_TYPE),
@@ -1681,7 +1677,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("popup_window", popup_window, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("prev_tagged_header", slrn_prev_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("quit", quit, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_0("raw_article_as_string", raw_article_as_string, SLANG_STRING_TYPE),
+   MAKE_INTRINSIC_0("raw_article_as_string", raw_article_as_string, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_article", re_bsearch_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_author", re_author_search_backward, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_subject", re_subject_search_backward, SLANG_INT_TYPE),
@@ -1740,30 +1736,44 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 #undef S
 #undef I
 
-static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
+static SLang_IConstant_Type Module_IConstants [] =
 {
-   MAKE_VARIABLE("ATTR_BLINK", &Interp_Attr_Blink, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("ATTR_BOLD", &Interp_Attr_Bold, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("ATTR_REV", &Interp_Attr_Rev, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("ATTR_ULINE", &Interp_Attr_Uline, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("GROUPS_DIRTY", &Slrn_Groups_Dirty, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("GROUP_NEW_GROUP_FLAG", &Interp_Group_New_Group_Flag, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("GROUP_UNSUBSCRIBED", &Interp_Group_Unsubscribed, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("HEADER_HIGH_SCORE", &Interp_Header_High_Score, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("HEADER_LOW_SCORE", &Interp_Header_Low_Score, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("HEADER_READ", &Interp_Header_Read, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version", &Slrn_Version, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version_String, SLANG_STRING_TYPE, 1),
-#ifdef HAVE_SETLOCALE
-# ifdef LC_CTYPE
-   MAKE_VARIABLE("LC_CTYPE", &Interp_Lc_Ctype, SLANG_INT_TYPE, 1),
-# endif
-# ifdef LC_TIME
-   MAKE_VARIABLE("LC_TIME", &Interp_Lc_Time, SLANG_INT_TYPE, 1),
-# endif
+   MAKE_ICONSTANT("ATTR_BLINK", SLTT_BLINK_MASK),
+   MAKE_ICONSTANT("ATTR_BOLD", SLTT_BOLD_MASK),
+   MAKE_ICONSTANT("ATTR_REV", SLTT_REV_MASK),
+   MAKE_ICONSTANT("ATTR_ULINE", SLTT_ULINE_MASK),
+   MAKE_ICONSTANT("GROUP_NEW_GROUP_FLAG", GROUP_NEW_GROUP_FLAG),
+   MAKE_ICONSTANT("GROUP_UNSUBSCRIBED", GROUP_UNSUBSCRIBED),
+   MAKE_ICONSTANT("HEADER_READ", HEADER_READ),
+   MAKE_ICONSTANT("HEADER_TAGGED", HEADER_TAGGED),
+   MAKE_ICONSTANT("HEADER_HIGH_SCORE", HEADER_HIGH_SCORE),
+   MAKE_ICONSTANT("HEADER_LOW_SCORE", HEADER_LOW_SCORE),
+   MAKE_ICONSTANT("HEADER_REQUEST_BODY", HEADER_REQUEST_BODY),
+   MAKE_ICONSTANT("HEADER_DONT_DELETE_MASK", HEADER_DONT_DELETE_MASK),
+   MAKE_ICONSTANT("HEADER_WITHOUT_BODY", HEADER_WITHOUT_BODY),
+   MAKE_ICONSTANT("HEADER_HIDDEN", HEADER_HIDDEN),
+   MAKE_ICONSTANT("HEADER_NTAGGED", HEADER_NTAGGED),
+   MAKE_ICONSTANT("FAKE_PARENT", FAKE_PARENT),
+   MAKE_ICONSTANT("FAKE_CHILDREN", FAKE_CHILDREN),
+   MAKE_ICONSTANT("FAKE_HEADER_HIGH_SCORE", FAKE_HEADER_HIGH_SCORE),
+   MAKE_ICONSTANT("HEADER_CHMAP_PROCESSED", HEADER_CHMAP_PROCESSED),
+   MAKE_ICONSTANT("HEADER_HAS_PARSE_PROBLEMS", HEADER_HAS_PARSE_PROBLEMS),
+   MAKE_ICONSTANT("HEADER_PROCESSED", HEADER_PROCESSED),
+#ifdef LC_CTYPE
+   MAKE_ICONSTANT("LC_CTYPE", LC_CTYPE),
+#endif
+#ifdef LC_TIME
+   MAKE_ICONSTANT("LC_TIME", LC_TIME),
 #endif
 
+   SLANG_END_ICONST_TABLE
+};
+
+static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
+{
+   MAKE_VARIABLE("GROUPS_DIRTY", &Slrn_Groups_Dirty, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version", &Slrn_Version, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version_String, SLANG_STRING_TYPE, 1),
    MAKE_VARIABLE(NULL, NULL, 0, 0)
 };
 
@@ -1832,6 +1842,7 @@ int slrn_init_slang (void) /*{{{*/
        /* Now add intrinsics for this application */
        || (-1 == SLadd_intrin_fun_table(Slrn_Intrinsics, NULL))
        || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL))
+       || (-1 == SLadd_iconstant_table (Module_IConstants, NULL))
        || (-1 == add_intrinsic_variables ()))
      return -1;
 
