@@ -1,7 +1,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
  Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
@@ -16,7 +16,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 /* post an article */
@@ -1908,7 +1908,7 @@ static int postpone_file (char *file) /*{{{*/
 	if (-1 == slrn_dircat (dir, NULL, dirfile, sizeof (dirfile)))
 	  return -1;
 
-	if (-1 == slrn_read_filename (_("Save to: "), NULL, dirfile, 1, 1))
+	if (-1 == slrn_read_filename (_("Save to: "), NULL, dirfile, 1, -1))
 	  return -1;
 
 	status = slrn_file_exists (dirfile);
