@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -17,7 +17,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -27,7 +27,6 @@
 #include <signal.h>
 #include <string.h>
 
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -73,6 +72,7 @@
 
 #include <errno.h>
 
+#include "common.h"
 #include "util.h"
 #include "server.h"
 #include "slrn.h"
@@ -82,11 +82,12 @@
 #include "art.h"
 #include "score.h"
 #include "snprintf.h"
-#include "chmap.h"
+#include "charset.h"
 #include "post.h"
 #include "xover.h"
 #include "mime.h"
 #include "hooks.h"
+#include "strutil.h"
 
 #if SLRN_USE_SLTCP
 # include "sltcp.h"
@@ -95,9 +96,7 @@
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
-#if SLRN_HAS_SLANG
-# include "interp.h"
-#endif
+#include "interp.h"
 #ifdef __os2__
 # define INCL_VIO
 # include <os2.h>
@@ -111,13 +110,22 @@
 
 /*{{{ Global Variables */
 
+#if SLANG_VERSION >= 20000
+int Slrn_UTF8_Mode = 0;
+#endif
+
 int Slrn_TT_Initialized = 0;
 
-/* If -1, force mouse.  If 1 the mouse will be used on in XTerm.  If 0, 
+/* If -1, force mouse.  If 1 the mouse will be used on in XTerm.  If 0,
  * do not use it.
  */
 int Slrn_Use_Mouse;
 
+/* Find out whether there was a warning at startup. Used for option -w0 */
+int Slrn_Saw_Warning = 0;
+
+/* We do not (yet) offer a batch mode; however, I consider it an interesting
+ * idea and might implement it after version 1.0.*/
 int Slrn_Batch;
 int Slrn_Suspension_Ok;
 int Slrn_Simulate_Graphic_Chars = 0;
@@ -130,12 +138,13 @@ int Slrn_Default_Post_Obj = SLRN_DEFAULT_POST_OBJ;
 
 FILE *Slrn_Debug_Fp = NULL;
 
-int Graphic_LTee_Char = SLSMG_LTEE_CHAR;
-int Graphic_UTee_Char = SLSMG_UTEE_CHAR;
-int Graphic_LLCorn_Char = SLSMG_LLCORN_CHAR;
-int Graphic_HLine_Char = SLSMG_HLINE_CHAR;
-int Graphic_VLine_Char = SLSMG_VLINE_CHAR;
-int Graphic_ULCorn_Char = SLSMG_ULCORN_CHAR;
+/* You need to call slrn_init_graphic_chars before using these */
+SLwchar_Type Graphic_LTee_Char;
+SLwchar_Type Graphic_UTee_Char;
+SLwchar_Type Graphic_LLCorn_Char;
+SLwchar_Type Graphic_HLine_Char;
+SLwchar_Type Graphic_VLine_Char;
+SLwchar_Type Graphic_ULCorn_Char;
 
 int Graphic_Chars_Mode = ALT_CHAR_SET_MODE;
 
@@ -147,6 +156,7 @@ static volatile int Want_Suspension;
 static volatile int Want_Window_Size_Change;
 static void perform_suspend (int);
 static int Current_Mouse_Mode;
+static int Ran_Startup_Hook;
 
 /*}}}*/
 /*{{{ Static Function Declarations */
@@ -166,7 +176,7 @@ static void test_lock( char *file ) /*{{{*/
 {
    int pid;
    FILE *fp;
-   
+
    if ((fp = fopen (file, "r")) != NULL)
      {
 	if (1 == fscanf (fp, "%d", &pid) )
@@ -180,7 +190,7 @@ static void test_lock( char *file ) /*{{{*/
 #if defined(IBMPC_SYSTEM)
 		  slrn_exit_error (_("\
 slrn: pid %d is locking the newsrc file. If you're not running another\n\
-      copy of slrn, delete the file %s"), 
+      copy of slrn, delete the file %s"),
 				   pid, file);
 #else
 		  slrn_exit_error (_("slrn: pid %d is locking the newsrc file."), pid);
@@ -197,15 +207,15 @@ static int make_lock( char *file ) /*{{{*/
 {
    int pid;
    FILE *fp;
-   
+
 #ifdef VMS
    fp = fopen (file, "w", "fop=cif");
 #else
    fp = fopen (file, "w");
 #endif
-   
+
    if (fp == NULL) return -1;
-   
+
    pid = getpid ();
    if (EOF == fprintf (fp, "%d", pid))
      {
@@ -225,10 +235,10 @@ static void lock_file (int how) /*{{{*/
    int rnlock = 0;
    char file_rn[SLRN_MAX_PATH_LEN];
 #endif
-   
+
    if (Slrn_Newsrc_File == NULL) return;
    if (not_ok_to_unlock) return;
-   
+
    not_ok_to_unlock = 1;
 
 #ifdef SLRN_USE_OS2_FAT
@@ -236,7 +246,7 @@ static void lock_file (int how) /*{{{*/
 #else
    slrn_snprintf (name, sizeof (name), "%s-lock", Slrn_Newsrc_File);
 #endif
-   
+
 #if SLRN_HAS_RNLOCK
    slrn_make_home_filename (".newsrc", file_rn, sizeof (file_rn));
    if (0 == strcmp (file_rn, Slrn_Newsrc_File))
@@ -256,14 +266,14 @@ static void lock_file (int how) /*{{{*/
 	  {
 	     slrn_exit_error (_("Unable to create lock file %s."), name);
 	  }
-	
+
 #if SLRN_HAS_RNLOCK
 	if (rnlock && (-1 == make_lock (file_rn)))
 	  {
 	     slrn_delete_file (name); /* delete the "normal" lock file */
 	     slrn_exit_error (_("Unable to create lock file %s."), file_rn);
 	  }
-#endif       
+#endif
      }
    else
      {
@@ -286,12 +296,12 @@ static void lock_file (int how) /*{{{*/
 /*}}}*/
 
 /*{{{ Signal Related Functions */
-	
+
 /*{{{ Low-Level signal-related utility functions */
 
 static void init_like_signals (int argc, int *argv, /*{{{*/
-			       void (*f0)(int), 
-			       void (*f1)(int), 
+			       void (*f0)(int),
+			       void (*f1)(int),
 			       int state)
 {
 #ifdef HAVE_SIGACTION
@@ -319,7 +329,7 @@ static void init_like_signals (int argc, int *argv, /*{{{*/
 	       {
 		  if (j != i) sigaddset (&sa.sa_mask, argv[j]);
 	       }
-	     
+
 	     (void) sigaction (sig, &sa, NULL);
 	  }
 #endif
@@ -328,8 +338,6 @@ static void init_like_signals (int argc, int *argv, /*{{{*/
 
 /*}}}*/
 
-
-
 /*}}}*/
 
 /*{{{ Suspension signals */
@@ -351,9 +359,9 @@ void slrn_push_suspension (int ok) /*{{{*/
 	Suspend_Stack [Suspension_Stack_Depth] = Ok_To_Suspend;
      }
    else ok = 0;
-   
+
    Suspension_Stack_Depth++;
-   
+
    (void) slrn_handle_interrupts ();
 
    Ok_To_Suspend = ok;
@@ -365,27 +373,27 @@ void slrn_push_suspension (int ok) /*{{{*/
 void slrn_pop_suspension (void) /*{{{*/
 {
 #ifdef REAL_UNIX_SYSTEM
-   
+
    if (Suspension_Stack_Depth == 0)
      {
 	slrn_error (_("pop_suspension: underflow!"));
 	return;
      }
-   
+
    Suspension_Stack_Depth--;
-   
+
    if (Suspension_Stack_Depth < SUSPEND_STACK_SIZE)
      {
 	Ok_To_Suspend = Suspend_Stack [Suspension_Stack_Depth];
      }
    else Ok_To_Suspend = 0;
 
-   (void) slrn_handle_interrupts ();   
+   (void) slrn_handle_interrupts ();
 #endif
 }
 /*}}}*/
 
-/* This function is called by the SIGTSTP handler.  Since it operates 
+/* This function is called by the SIGTSTP handler.  Since it operates
  * in an asynchronous fashion, care must be exercised to control when that
  * can happen.  This is accomplished via the push/pop_suspension functions.
  */
@@ -393,14 +401,14 @@ static void sig_suspend (int sig)
 {
 #ifdef REAL_UNIX_SYSTEM
    sig = errno;
-   
-   if (Ok_To_Suspend 
+
+   if (Ok_To_Suspend
        && (0 == Suspend_Sigtstp_Suspension))
      {
 	perform_suspend (1);
      }
    else Want_Suspension = 1;
-   
+
    init_suspend_signals (1);
    errno = sig;
 #else
@@ -412,14 +420,14 @@ static void init_suspend_signals (int state) /*{{{*/
 {
    int argv[2];
    int argc = 0;
-   
+
    if (Can_Suspend == 0)
      return;
 
 #ifdef SIGTSTP
    argv[argc++] = SIGTSTP;
 #endif
-   
+
 #ifdef SIGTTIN
    argv[argc++] = SIGTTIN;
 #endif
@@ -436,7 +444,7 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
    slrn_error (_("Not implemented."));
    Want_Suspension = 0;
 #else
-   
+
    int init;
    int mouse_mode = Current_Mouse_Mode;
 
@@ -449,30 +457,30 @@ static void perform_suspend (int smg_suspend_flag) /*{{{*/
 	slrn_error (_("Suspension not allowed by shell."));
 	return;
      }
-   
+
    sigemptyset (&mask);
    sigaddset (&mask, SIGTSTP);
-   
+
    /* This function resets SIGTSTP to default */
    init = suspend_display_mode (smg_suspend_flag);
-   
+
    kill (getpid (), SIGTSTP);
-   
+
    /* If SIGTSTP is pending, it will be delivered now.  That's ok. */
    sigprocmask (SIG_UNBLOCK, &mask, NULL);
 # else
-   
+
    Want_Suspension = 0;
    if (Can_Suspend == 0)
      {
 	slrn_error (_("Suspension not allowed by shell."));
 	return;
      }
-   
+
    init = suspend_display_mode (smg_suspend_flag);
    kill(getpid(),SIGSTOP);
 # endif
-   
+
    resume_display_mode (smg_suspend_flag, init, mouse_mode);
 #endif				       /* !defined(SIGSTOP) || !defined(REAL_UNIX_SYSTEM) */
 }
@@ -488,16 +496,16 @@ static void check_for_suspension (void)
 {
 #ifdef SIGTSTP
    void (*f)(int);
-   
+
    f = SLsignal (SIGTSTP, SIG_DFL);
    (void) SLsignal (SIGTSTP, f);
-   
+
 #if 0
    Can_Suspend = (f == SIG_DFL);
 #else
    Can_Suspend = (f != SIG_IGN);
 #endif
-   
+
 #else
    Can_Suspend = 0;
 #endif
@@ -507,10 +515,9 @@ static void check_for_suspension (void)
 
 /*{{{ Hangup Signals */
 
-
 void slrn_init_hangup_signals (int state) /*{{{*/
 {
-   int argv[2];
+   int argv[3];
    int argc = 0;
 
 #ifdef SIGHUP
@@ -519,7 +526,10 @@ void slrn_init_hangup_signals (int state) /*{{{*/
 #ifdef SIGTERM
    argv[argc++] = SIGTERM;
 #endif
-   
+#ifdef SIGQUIT
+   argv[argc++] = SIGQUIT;
+#endif
+
    init_like_signals (argc, argv, SIG_IGN, slrn_hangup, state);
 }
 
@@ -541,10 +551,10 @@ static void sig_winch_handler (int sig)
 static void slrn_set_screen_size (int sig) /*{{{*/
 {
    int old_r, old_c;
-   
+
    old_r = SLtt_Screen_Rows;
    old_c = SLtt_Screen_Cols;
-   
+
    SLtt_get_screen_size ();
 
    Slrn_Full_Screen_Update = 1;
@@ -569,7 +579,7 @@ static void slrn_set_screen_size (int sig) /*{{{*/
 static void init_display_signals (int mode) /*{{{*/
 {
    init_suspend_signals (mode);
-   
+
    if (mode)
      {
 	SLang_set_abort_signal (NULL);
@@ -579,7 +589,7 @@ static void init_display_signals (int mode) /*{{{*/
 #ifdef SIGTTOU
 	/* Allow background writes */
 	SLsignal (SIGTTOU, SIG_IGN);
-#endif   
+#endif
 #ifdef SIGWINCH
 	SLsignal_intr (SIGWINCH, sig_winch_handler);
 #endif
@@ -600,16 +610,15 @@ int slrn_handle_interrupts (void)
      {
 	slrn_suspend_cmd ();
      }
-   
+
    if (Want_Window_Size_Change)
      {
 	slrn_set_screen_size (1);
      }
-   
+
    return 0;
 }
 
-
 /*}}}*/
 
 /*{{{ Screen Management and Terminal Init/Reset Functions */
@@ -621,7 +630,7 @@ static int init_tty (void) /*{{{*/
      {
 	return 0;
      }
-   
+
    if (Slrn_TT_Initialized == 0)
      init_display_signals (1);
 
@@ -636,9 +645,9 @@ static int init_tty (void) /*{{{*/
    if (Can_Suspend)
      SLtty_set_suspend_state (1);
 #endif
-   
+
    Slrn_TT_Initialized |= SLRN_TTY_INIT;
-   
+
    return 0;
 }
 
@@ -650,13 +659,13 @@ static int reset_tty (void) /*{{{*/
      {
 	return 0;
      }
-   
+
    SLang_reset_tty ();
    Slrn_TT_Initialized &= ~SLRN_TTY_INIT;
-   
+
    if (Slrn_TT_Initialized == 0)
      init_display_signals (0);
-   
+
    return 0;
 }
 
@@ -671,7 +680,7 @@ static int init_smg (int use_resume, int mouse_mode) /*{{{*/
 
    if (Slrn_TT_Initialized == 0)
      init_display_signals (1);
-      
+
    if (use_resume)
      {
 	SLsmg_resume_smg ();
@@ -682,14 +691,14 @@ static int init_smg (int use_resume, int mouse_mode) /*{{{*/
 	slrn_set_screen_size (0);
 	SLsmg_init_smg ();
 	Slrn_TT_Initialized |= SLRN_SMG_INIT;
-	     
+
 	/* We do not want the -> overlay cursor to affect the scroll. */
 #if !defined(IBMPC_SYSTEM)
 	SLsmg_Scroll_Hash_Border = 5;
 #endif
 	slrn_redraw ();
      }
-      
+
    return 0;
 }
 
@@ -698,10 +707,10 @@ static int init_smg (int use_resume, int mouse_mode) /*{{{*/
 static int reset_smg (int smg_suspend_flag) /*{{{*/
 {
    if (0 == (Slrn_TT_Initialized & SLRN_SMG_INIT))
-     return 0; 
-   
+     return 0;
+
    slrn_enable_mouse (0);
-   
+
    if (smg_suspend_flag)
      SLsmg_suspend_smg ();
    else
@@ -710,14 +719,14 @@ static int reset_smg (int smg_suspend_flag) /*{{{*/
 	slrn_smg_refresh ();
 	SLsmg_reset_smg ();
      }
-   
+
    /* SLsignal_intr (SIGWINCH, SIG_DFL); */
-   
+
    Slrn_TT_Initialized &= ~SLRN_SMG_INIT;
 
    if (Slrn_TT_Initialized == 0)
      init_display_signals (0);
-   
+
    return 0;
 }
 
@@ -729,10 +738,10 @@ static int suspend_display_mode (int smg_suspend_flag) /*{{{*/
    int mode = Slrn_TT_Initialized;
 
    SLsig_block_signals ();
-   
+
    reset_smg (smg_suspend_flag);
    reset_tty ();
-   
+
    SLsig_unblock_signals ();
 
    return mode;
@@ -743,13 +752,13 @@ static int suspend_display_mode (int smg_suspend_flag) /*{{{*/
 static int resume_display_mode (int smg_suspend_flag, int mode, int mouse_mode) /*{{{*/
 {
    SLsig_block_signals ();
-   
+
    if (mode & SLRN_TTY_INIT)
      init_tty ();
-   
+
    if (mode & SLRN_SMG_INIT)
      init_smg (smg_suspend_flag, mouse_mode);
-   
+
    SLsig_unblock_signals ();
    return 0;
 }
@@ -760,19 +769,19 @@ static int resume_display_mode (int smg_suspend_flag, int mode, int mouse_mode)
 void slrn_set_display_state (int state) /*{{{*/
 {
    if (Slrn_Batch) return;
-   
+
    SLsig_block_signals ();
-   
+
    if (state & SLRN_TTY_INIT)
      init_tty ();
    else
      reset_tty ();
-   
+
    if (state & SLRN_SMG_INIT)
      init_smg (0, 1);
    else
      reset_smg (0);
-   
+
    SLsig_unblock_signals ();
 }
 
@@ -799,7 +808,7 @@ void slrn_init_graphic_chars (void) /*{{{*/
    if (SLtt_Has_Alt_Charset == 0)
      Slrn_Simulate_Graphic_Chars = 1;
 #endif
-   
+
    if (Slrn_Simulate_Graphic_Chars)
      {
 	Graphic_LTee_Char = '+';
@@ -808,7 +817,7 @@ void slrn_init_graphic_chars (void) /*{{{*/
 	Graphic_HLine_Char = '-';
 	Graphic_VLine_Char = '|';
 	Graphic_ULCorn_Char = '/';
-	
+
 	Graphic_Chars_Mode = SIMULATED_CHAR_SET_MODE;
      }
    else
@@ -820,8 +829,8 @@ void slrn_init_graphic_chars (void) /*{{{*/
 	Graphic_HLine_Char = SLSMG_HLINE_CHAR;
 	Graphic_VLine_Char = SLSMG_VLINE_CHAR;
 	Graphic_ULCorn_Char = SLSMG_ULCORN_CHAR;
-	
-	Graphic_Chars_Mode = ALT_CHAR_SET_MODE;   
+
+	Graphic_Chars_Mode = ALT_CHAR_SET_MODE;
      }
 }
 /*}}}*/
@@ -830,11 +839,11 @@ void slrn_init_graphic_chars (void) /*{{{*/
 static void perform_cleanup (void)
 {
    slrn_set_display_state (0);
-   
+
 #if SLRN_HAS_GROUPLENS
    slrn_close_grouplens ();
 #endif
-      
+
    if (Slrn_Server_Obj != NULL)
      Slrn_Server_Obj->sv_close ();
 
@@ -843,14 +852,14 @@ static void perform_cleanup (void)
 	(void) fclose (Slrn_Debug_Fp);
 	Slrn_Debug_Fp = NULL;
      }
-   
+
 #if SLRN_HAS_GROUPLENS
    slrn_close_grouplens ();
 #endif
 
-#if SLRN_HAS_SLANG
+   if (Ran_Startup_Hook)
+     (void) slrn_run_hooks (HOOK_QUIT, 0);
    (void) slrn_reset_slang ();
-#endif
 
 #if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
    (void) sltcp_close_sltcp ();
@@ -875,7 +884,7 @@ void slrn_va_exit_error (char *fmt, va_list ap)
      {
 	trying_to_exit = 1;
 	slrn_set_display_state (0);
-	   
+
 	if (fmt != NULL)
 	  {
 	     fputs (_("slrn fatal error:"), stderr);
@@ -884,7 +893,7 @@ void slrn_va_exit_error (char *fmt, va_list ap)
 #endif
 	     fputc ('\n', stderr);
 	     vfprintf (stderr, fmt, ap);
-	  }   
+	  }
 	if (Slrn_Groups_Dirty) slrn_write_newsrc (0);
 	perform_cleanup ();
      }
@@ -896,7 +905,6 @@ void slrn_va_exit_error (char *fmt, va_list ap)
 void slrn_exit_error (char *fmt, ...) /*{{{*/
 {
    va_list ap;
-   
    va_start (ap, fmt);
    slrn_va_exit_error (fmt, ap);
    va_end(ap);
@@ -904,7 +912,18 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
-static void usage (char *extra) /*{{{*/
+void slrn_error (char *fmt, ...) /*{{{*/
+{
+   va_list ap;
+
+   va_start(ap, fmt);
+   slrn_verror (fmt, ap);
+   va_end (ap);
+}
+
+/*}}}*/
+
+static void usage (char *extra, int exit_status) /*{{{*/
 {
    printf (_("\
 Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
@@ -922,47 +941,48 @@ Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
 -n              Do not check for new groups.  This usually results in\n\
                  a faster startup.\n\
 -w              Wait for key before switching to full screen mode\n\
+-w0             Wait for key (only when warnings / errors occur)\n\
 --create        Create a newsrc file by getting list of groups from server.\n\
 --debug FILE    Write debugging information to FILE\n\
 --help          Print this usage.\n\
 --kill-log FILE Keep a log of all killed articles in FILE\n\
+--show-config   Print configuration\n\
 --version       Show version and supported features\n\
 \n\
 NNTP mode has additional options; use \"slrn --nntp --help\" to display them.\n\
 "), SLRN_USER_SLRNRC_FILENAME);
-   
+
    if (extra != NULL)
      {
 	printf ("\n%s\n", extra);
      }
-   exit (1);
+   exit (exit_status);
 }
 
 /*}}}*/
 
-
 static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
 {
    int num_parsed;
    int zero_ok = 1;
-   
+
    if (obj == NULL)
      {
 	zero_ok = 0;
 	obj = slrn_map_object_id_to_name (0, SLRN_DEFAULT_SERVER_OBJ);
      }
-   
+
    num_parsed = slrn_parse_object_args (obj, argv, argc);
    if (num_parsed < 0)
      {
 	if (num_parsed == -1)
 	  slrn_exit_error (_("%s is not a supported option."), *argv);
-	else 
+	else
 	  slrn_exit_error (_("%s is not supported."), obj);
      }
    if ((num_parsed == 0) && (zero_ok == 0))
-     usage (NULL);
-   
+     usage (NULL, 1);
+
    return num_parsed;
 }
 
@@ -971,15 +991,16 @@ static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
 static void read_score_file (void)
 {
    char file[SLRN_MAX_PATH_LEN];
-   
+
    if (Slrn_Score_File == NULL)
      return;
-   
+
    slrn_make_home_filename (Slrn_Score_File, file, sizeof (file));
-   
+
    if (1 != slrn_file_exists (file))
      {
 	slrn_message_now (_("*** Warning: Score file %s does not exist"), file);
+	Slrn_Saw_Warning = 1;
 	return;
      }
 
@@ -989,7 +1010,6 @@ static void read_score_file (void)
      }
 }
 
-
 static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 {
    char *hlp_file;
@@ -1006,40 +1026,20 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    char file [SLRN_MAX_PATH_LEN];
    char *init_file = NULL;
    char *kill_logfile = NULL;
+   int print_config = 0;
 
 #if defined(HAVE_SETLOCALE) && defined(LC_ALL)
    (void) setlocale(LC_ALL, "");
 #endif
 #ifdef ENABLE_NLS
-   bindtextdomain(PACKAGE,LOCALEDIR);
-   textdomain(PACKAGE);
+   bindtextdomain(NLS_PACKAGE_NAME, NLS_LOCALEDIR);
+   textdomain (NLS_PACKAGE_NAME);
 #endif
 
    check_for_suspension ();
 #ifdef __unix__
    (void) umask (077);
 #endif
-   
-   /* These are already available if the interpreter is available */
-#if !SLRN_HAS_SLANG
-# ifdef __os2__
-   SLdefine_for_ifdef ("OS2");
-# else
-#  ifdef VMS
-   SLdefine_for_ifdef ("VMS");
-#  else
-#   ifdef __NT__
-   SLdefine_for_ifdef ("NT");
-#   else
-#    ifdef __WIN32__
-   SLdefine_for_ifdef ("WIN32");
-#    else
-   SLdefine_for_ifdef ("UNIX");
-#    endif
-#   endif
-#  endif
-# endif
-#endif
 
 #if 0
    if (NULL != getenv ("AUTOSUBSCRIBE"))
@@ -1047,17 +1047,17 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if (NULL != getenv ("AUTOUNSUBSCRIBE"))
      Slrn_Unsubscribe_New_Groups = 1;
 #endif
-   
+
    for (i = 1; i < (unsigned int) argc; i++)
      {
 	char *argv_i = argv[i];
-	
+
 	if ((argv_i[0] == '-') && (argv_i[1] == '-'))
 	  {
 	     int status;
 
 	     argv_i += 2;
-	     
+
 	     status = slrn_parse_object_args (argv_i, NULL, 0);
 	     if (status != -1)
 	       i += parse_object_args (argv_i, argv + (i + 1), argc - (i + 1));
@@ -1066,7 +1066,12 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 #endif
 	     else if (!strcmp ("create", argv_i)) create_flag = 1;
 	     else if (!strcmp ("version", argv_i))
-	       slrn_show_version ();
+	       {
+		  slrn_show_version (stdout);
+		  exit (0);
+	       }
+	     else if (!strcmp ("show-config", argv_i))
+	       print_config = 1;
 	     else if ((!strcmp ("kill-log", argv_i)) &&
 		      (i + 1 < (unsigned int) argc))
 	       kill_logfile = argv[++i];
@@ -1080,7 +1085,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
                     slrn_exit_error (_("Unable to open %s for debugging."), argv[i]);
                   setbuf (Slrn_Debug_Fp, (char *) NULL);
                }
-	     else usage (NULL);
+	     else if (!strcmp ("help", argv_i))
+	       usage (NULL, 0);
+	     else usage (NULL, 1);
 	  }
 	else if (!strcmp ("-create", argv_i)) create_flag = 1;
 	else if (!strcmp ("-C", argv_i)) use_color = 1;
@@ -1094,6 +1101,8 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	  Slrn_Perform_Scoring &= ~SLRN_EXPENSIVE_SCORING;
 	else if (!strcmp ("-w", argv_i))
 	  wait_for_key = 1;
+	else if (!strcmp ("-w0", argv_i))
+	  wait_for_key = 2;
 	else if (!strncmp ("-D", argv_i, 2) && (argv_i[2] != 0))
 	  {
 	     if (-1 == SLdefine_for_ifdef (argv_i + 2))
@@ -1106,48 +1115,52 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	  {
 	     if (!strcmp ("-f", argv_i)) Slrn_Newsrc_File = argv[++i];
 	     else if (!strcmp ("-i", argv_i)) init_file = argv[++i];
-	     else 
+	     else
 	       {
 		  i += parse_object_args (NULL, argv + i, argc - i);
 		  i -= 1;
 	       }
 	  }
-	else 
+	else
 	  {
 	     i += parse_object_args (NULL, argv + i, argc - i);
 	     i -= 1;
 	  }
      }
-   
-   fprintf (stdout, "slrn %s [%s]\n", Slrn_Version, Slrn_Date);
+
+   fprintf (stdout, "slrn %s\n", Slrn_Version_String);
 
    if (dsc_flag && create_flag)
      {
-	usage (_("The -d and --create flags must not be specified together."));
+	usage (_("The -d and --create flags must not be specified together."), 1);
      }
-   
+
    if (Slrn_Batch == 0)
      {
+	Slrn_UTF8_Mode = SLutf8_enable (-1);
 	SLtt_get_terminfo ();
 	if (use_color == 1) SLtt_Use_Ansi_Colors = 1;
 	else if (use_color == -1) SLtt_Use_Ansi_Colors = 0;
      }
 
 #if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
+   SLtcp_Verror_Hook = slrn_verror;
    /* This is necessary to ensure that gethostname works on Win32 */
    if (-1 == sltcp_open_sltcp ())
      slrn_exit_error (_("Error initializing SLtcp interface"));
 #endif
 
-#if SLRN_HAS_SLANG
    if (-1 == slrn_init_slang ())
-     fprintf (stderr, _("Error initializing S-Lang interpreter.\n"));
-#endif
+     slrn_exit_error (_("Error initializing S-Lang interpreter.\n"));
 
    slrn_startup_initialize ();
 
-   /* The next function call will also define slang preprocessing tokens 
-    * for the appropriate objects.  For that reason, it is called after 
+   /* We need to get user info first, because the request file in true offline
+    * reading mode is chosen based on login name */
+   slrn_get_user_info ();
+
+   /* The next function call will also define slang preprocessing tokens
+    * for the appropriate objects.  For that reason, it is called after
     * startup initialize.
     */
    if (-1 == slrn_init_objects ())
@@ -1155,33 +1168,31 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	slrn_exit_error (_("Error configuring server objects."));
      }
 
-   slrn_get_user_info ();
-
    slrn_init_hangup_signals (1);
 
 #ifdef VMS
-   slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s%s", SLRN_CONF_DIR, "slrn.rc");
 #else
-   slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s/%s", SLRN_CONF_DIR, "slrn.rc");
 #endif
 
-   /* Make sure terminal is initialized before setting colors.  The 
+   /* Make sure terminal is initialized before setting colors.  The
     * SLtt_get_terminfo call above fixed that.
     */
    slrn_read_startup_file (file);      /* global file for all users */
-   
+
    if (init_file == NULL)
      {
 	slrn_make_home_filename (SLRN_USER_SLRNRC_FILENAME, file, sizeof (file));
 	init_file = file;
      }
-   
+
    if ((-1 == slrn_read_startup_file (init_file)) && (init_file != file))
      {
 	slrn_exit_error (_("\
 Could not read specified config file %s\n"), init_file);
      }
-   
+
    if (Slrn_Saw_Obsolete)
      {
 	slrn_message (_("\n! Your configuration file contains obsolete commands or function names that\n"
@@ -1189,47 +1200,28 @@ Could not read specified config file %s\n"), init_file);
 		      "! If you have Perl installed, you can use the script slrnrc-conv to change\n"
 		      "! your configuration accordingly.  It can be found in the source distribution\n"
 		      "! or retrieved from <URL:http://slrn.sourceforge.net/data/>.\n"));
+	Slrn_Saw_Warning = 1;
      }
 
    if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
    if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
    if (no_new_groups) Slrn_Check_New_Groups = 0;
 
-#if SLRN_HAS_CHARACTER_MAP
-   if (-1 == slrn_set_charset (Slrn_Charset))
-     {
-	slrn_chmap_show_supported ();
-	slrn_exit_error (_("Failed to select character set."));
-     }
-#endif
+   slrn_prepare_charset();
 
-#if SLRN_HAS_MIME
-   if (NULL == Slrn_Mime_Display_Charset)
+   if (print_config)
      {
-# if SLRN_HAS_CHARACTER_MAP
-	if (NULL != Slrn_Charset)
-	  {
-	     if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					(unsigned char *)"ibm852"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-2");
-	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					     (unsigned char *)"koi8"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("koi8-r");
-	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					     (unsigned char *)"ibm737"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-7");
-	  }
-	if (NULL == Slrn_Mime_Display_Charset)
-# endif
-	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-1");
+	(void) putc ('\n', stdout);
+	slrn_show_version (stdout);
+	slrn_print_config (stdout);
+	slrn_quit (0);
      }
-#endif
 
 #ifdef SIGINT
    if (Slrn_TT_Initialized == 0)
      SLsignal_intr (SIGINT, SIG_DFL);
 #endif
-   
+
    if ((-1 == slrn_select_server_object (Slrn_Server_Id))
        || (-1 == slrn_select_post_object (Slrn_Post_Id)))
      {
@@ -1241,7 +1233,7 @@ Could not read specified config file %s\n"), init_file);
    if (SLtt_Use_Ansi_Colors == 0)
      SLtt_Blink_Mode = 1;
 #endif
-   
+
    /* Now that we have read in the startup file, check to see if the user
     * has a username and a usable hostname.  Without those, we are not
     * starting up.
@@ -1251,12 +1243,13 @@ Could not read specified config file %s\n"), init_file);
      {
 	slrn_exit_error (_("\
 Unable to find a valid hostname for constructing your e-mail address.\n\
-You will have to specify a hostname in your %s file.\n"), 
+You probably want to specify a hostname in your %s file.\n\
+Please see the \"slrn reference manual\" for full details.\n"),
 			 SLRN_USER_SLRNRC_FILENAME);
      }
    if ((NULL == Slrn_User_Info.username)
        || (0 == *Slrn_User_Info.username)
-       || (NULL != slrn_strchr (Slrn_User_Info.username, '@')))
+       || (NULL != slrn_strbyte (Slrn_User_Info.username, '@')))
      {
 	slrn_exit_error (_("\
 Unable to find your user name.  This means that a valid 'From' header line\n\
@@ -1265,26 +1258,29 @@ cannot be constructed.  Try setting the USER environment variable.\n"));
 
    if ((NULL == Slrn_User_Info.posting_host)
        && Slrn_Generate_Message_Id)
-     slrn_stderr_strcat (_("\
+     {
+	slrn_stderr_strcat (_("\
 ***Warning: Unable to find a unique fully-qualified host name."), "\n", _("\
             slrn will not generate any Message-IDs."), "\n", _("\
             Please note that the \"hostname\" setting does not affect this;"), "\n", _("\
             see the \"slrn reference manual\" for details."), "\n", NULL);
-   
+	Slrn_Saw_Warning = 1;
+     }
+
    if (no_score_file == 0) read_score_file ();
-   
+
    if (NULL == (hlp_file = getenv ("SLRNHELP")))
      {
 	hlp_file = file;
 #ifdef VMS
-	slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s%s", SLRN_CONF_DIR, "help.txt");
 #else
-	slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s/%s", SLRN_CONF_DIR, "help.txt");
 #endif
      }
-	
+
    slrn_parse_helpfile (hlp_file);
-   
+
    if ((Slrn_Newsrc_File == NULL)
        && ((Slrn_Newsrc_File = slrn_map_file_to_host (Slrn_Server_Obj->sv_name)) == NULL))
      {
@@ -1296,10 +1292,10 @@ cannot be constructed.  Try setting the USER environment variable.\n"));
 	slrn_make_home_filename (Slrn_Newsrc_File, file, sizeof (file));
 	Slrn_Newsrc_File = slrn_safe_strmalloc (file);
      }
-   
+
    slrn_message_now (_("Using newsrc file %s for server %s."),
 		     Slrn_Newsrc_File, Slrn_Server_Obj->sv_name);
-   
+
    if (use_active) Slrn_List_Active_File = 1;
    if (use_mouse) Slrn_Use_Mouse = -1; /* -1 forces it. */
    if (use_color == 1) SLtt_Use_Ansi_Colors = 1;
@@ -1315,7 +1311,7 @@ cannot be constructed.  Try setting the USER environment variable.\n"));
 	Slrn_Server_Obj->sv_close ();
 	exit (0);
      }
-   
+
    if (create_flag == 0)
      {
 	/* Check to see if the .newsrc file exists -- I should use the access
@@ -1338,24 +1334,22 @@ If you want to create %s, add command line options:\n\
 	  }
 	slrn_fclose (fp);
      }
-   
+
    /* make sure we have an entry for the server */
    slrn_add_to_server_list (Slrn_Server_Obj->sv_name, NULL, NULL, NULL);
-   
+
    if (kill_logfile != NULL)
      {
 	Slrn_Kill_Log_FP = fopen(kill_logfile, "a");
 	if (Slrn_Kill_Log_FP == NULL)
 	  slrn_error (_("Unable to open %s for logging."), kill_logfile);
      }
-   
+
    lock_file (1);
 
-#if SLRN_HAS_SLANG
    (void) slrn_run_hooks (HOOK_STARTUP, 0);
-#endif
-   
-   
+   Ran_Startup_Hook = 1;
+
 #if SLRN_HAS_GROUPLENS
    if (Slrn_Server_Id != SLRN_SERVER_ID_NNTP)
      Slrn_Use_Group_Lens = 0;
@@ -1367,6 +1361,7 @@ If you want to create %s, add command line options:\n\
 	  {
 	     fprintf (stderr, _("GroupLens disabled.\n"));
 	     Slrn_Use_Group_Lens = 0;
+	     Slrn_Saw_Warning = 1;
 	  }
      }
 #endif
@@ -1375,22 +1370,22 @@ If you want to create %s, add command line options:\n\
      {
 	slrn_exit_error (_("Failed to initialize server."));
      }
-   
+
    if (Slrn_Server_Obj->sv_has_xover == 1)
      {
 	if (0 == (Slrn_Server_Obj->sv_has_xover = slrn_read_overview_fmt ()))
 	  slrn_message (_("OVERVIEW.FMT not RFC 2980 compliant -- XOVER support disabled."));
      }
-   
+
    if (Slrn_Check_New_Groups || create_flag)
      {
 	slrn_get_new_groups (create_flag);
      }
-   
+
    slrn_read_newsrc (create_flag);
    slrn_read_group_descriptions ();
-   
-   if (wait_for_key)
+
+   if (wait_for_key && ((wait_for_key == 1) || Slrn_Saw_Warning))
      {
 	slrn_message (_("* Press Ctrl-C to quit, any other key to continue.\n"));
 	slrn_set_display_state (SLRN_TTY_INIT);
@@ -1399,20 +1394,18 @@ If you want to create %s, add command line options:\n\
      }
    else
      putc ('\n', stdout);
-   
+
    slrn_set_display_state (SLRN_SMG_INIT | SLRN_TTY_INIT);
-   
+
 #if defined(__unix__) && !defined(IBMPC_SYSTEM)
    if (Slrn_Autobaud) SLtt_Baud_Rate = SLang_TT_Baud_Rate;
 #endif
-   
+
    return 0;
 }
 
 /*}}}*/
 
-
-
 /*{{{ Main Loop and Key Processing Functions */
 #define MAX_MODE_STACK_LEN 15
 /* Allow one extra because the top one is the current mode */
@@ -1428,7 +1421,7 @@ void slrn_push_mode (Slrn_Mode_Type *mode)
    Mode_Stack_Depth++;
 
    Mode_Stack[Mode_Stack_Depth] = Slrn_Current_Mode = mode;
-   
+
    Slrn_Full_Screen_Update = 1;
    if ((mode != NULL) && (mode->enter_mode_hook != NULL))
      (*mode->enter_mode_hook) ();
@@ -1438,7 +1431,7 @@ void slrn_pop_mode (void)
 {
    if (Mode_Stack_Depth == 0)
      slrn_exit_error (_("Internal Error: Mode_Stack underflow"));
-   
+
    Mode_Stack [Mode_Stack_Depth] = NULL;   /* null current mode */
    Mode_Stack_Depth--;
    Slrn_Current_Mode = Mode_Stack[Mode_Stack_Depth];
@@ -1448,14 +1441,13 @@ void slrn_pop_mode (void)
      (*Slrn_Current_Mode->enter_mode_hook) ();
 }
 
-
 void slrn_update_screen (void)
 {
    Slrn_Mode_Type *mode;
    unsigned int i, imax;
 
    if (Slrn_Batch) return;
-   
+
    slrn_push_suspension (0);
    imax = Mode_Stack_Depth + 1;	       /* include current mode */
    for (i = 0; i < imax; i++)
@@ -1484,20 +1476,17 @@ static void run_winch_functions (int old_r, int old_c)
 	  (*mode->sigwinch_fun) (old_r, old_c);
      }
 
-#if SLRN_HAS_SLANG
-   if (SLang_Error == 0)
+   if (SLang_get_error() == 0)
      slrn_run_hooks (HOOK_RESIZE_SCREEN, 0);
-#endif
 }
 
-
 static void slrn_hangup (int sig) /*{{{*/
 {
    Slrn_Mode_Type *mode;
    unsigned int i;
 
    slrn_init_hangup_signals (0);
-   
+
    i = Mode_Stack_Depth + 1;	       /* include current */
    while (i != 0)
      {
@@ -1507,7 +1496,7 @@ static void slrn_hangup (int sig) /*{{{*/
 	    && (mode->hangup_fun != NULL))
 	  (*mode->hangup_fun) (sig);
      }
-   
+
    slrn_write_newsrc (0);
    slrn_quit (sig);
 }
@@ -1517,36 +1506,40 @@ static void slrn_hangup (int sig) /*{{{*/
 void slrn_call_command (char *cmd) /*{{{*/
 {
    SLKeymap_Function_Type *list;
-   
+
    if ((Slrn_Current_Mode == NULL)
        || (Slrn_Current_Mode->keymap == NULL))
      list = NULL;
    else
      list = Slrn_Current_Mode->keymap->functions;
-   
+
    while ((list != NULL) && (list->name != NULL))
      {
 	if (0 == strcmp (cmd, list->name))
 	  {
 	     (void) (*list->f) ();
+	     /* sync the line number to avoid surprises in subsequent calls
+	      * that might change the article window (not the article itself) */
+	     if (Slrn_Current_Mode->mode == SLRN_ARTICLE_MODE)
+	       slrn_art_sync_article (Slrn_Current_Article);
 	     return;
 	  }
 	list++;
      }
-   
+
    slrn_error (_("call: %s not in current keymap."), cmd);
 }
 
 /*}}}*/
 
-static int slrn_getkey (void)
+int slrn_getkey (void)
 {
    static char buf[32];
    static unsigned int buf_len;
    static int timeout_active;
-   
+
    int ch;
-   
+
    if (SLang_Key_TimeOut_Flag == 0)
      {
 	timeout_active = 0;
@@ -1556,23 +1549,44 @@ static int slrn_getkey (void)
 	    && (buf_len + 2 < sizeof (buf)))
      {
 	int r, c;
-	
+
 	buf[buf_len] = '-';
 	buf[buf_len + 1] = 0;
-	
+
 	slrn_push_suspension (0);
 	r = SLsmg_get_row (); c = SLsmg_get_column ();
-	slrn_message (buf);
+	slrn_message ("%s", buf);
 	SLsmg_gotorc (r, c);
 	slrn_smg_refresh ();
 	slrn_pop_suspension ();
 	timeout_active = 1;
      }
-   
-   ch = SLang_getkey ();
+
+   while (1)
+     {
+#if defined(REAL_UNIX_SYSTEM) && (SLANG_VERSION < 20202)
+	int ttyfd = SLang_TT_Read_FD;
+#endif
+	int e;
+	errno = 0;
+
+	ch = SLang_getkey ();
+	if (ch != SLANG_GETKEY_ERROR)
+	  break;
+	e = errno;
+
+#if defined(REAL_UNIX_SYSTEM) && (SLANG_VERSION < 20202)
+	if (ttyfd != SLang_TT_Read_FD)
+	  continue;
+#endif
+
+	slrn_exit_error ("%s: errno=%d [%s]", _("SLang_getkey failed"),
+			 e, SLerrno_strerror (e));
+     }
+
    if (buf_len + 4 < sizeof (buf))
      {
-	if (ch == 0) 
+	if (ch == 0)
 	  {
 	     /* Need to handle NULL character. */
 	     buf[buf_len++] = '^';
@@ -1588,46 +1602,43 @@ static int slrn_getkey (void)
 	else buf[buf_len] = (char) ch;
      }
    buf_len++;
-   
+
    return ch;
 }
 
-   
 void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
 {
    SLang_Key_Type *key;
    static SLKeyMap_List_Type *last_map;
    static SLang_Key_Type *last_key;
 
-   
    Suspend_Sigtstp_Suspension = 1;
    key = SLang_do_key (map, slrn_getkey);
    Suspend_Sigtstp_Suspension = 0;
 
-   if (Slrn_Message_Present || SLang_Error) 
+   if (Slrn_Message_Present || SLang_get_error())
      {
-#if SLRN_HAS_SLANG
-	if (SLang_Error) SLang_restart (0);
-#endif
+	if (SLang_get_error()) SLang_restart (0);
 	slrn_clear_message ();
      }
-   SLang_Error = SLKeyBoard_Quit = 0;
-   
+   SLKeyBoard_Quit = 0;
+   SLang_set_error (0);
+
    if ((key == NULL) || (key->type == 0))
      {
 	SLtt_beep ();
 	return;
      }
-   
+
    if (key->type == SLKEY_F_INTRINSIC)
      {
 	if ((map == last_map) && (key->f.f == (FVOID_STAR) slrn_repeat_last_key))
 	  key = last_key;
-	
+
 	/* set now to avoid problems with recursive call */
 	last_key = key;
 	last_map = map;
-	
+
 	if (key->type == SLKEY_F_INTRINSIC)
 	  {
 	     (((void (*)(void))(key->f.f)) ());
@@ -1636,17 +1647,14 @@ void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
      }
 
    /* Otherwise we have interpreted key. */
-#if SLRN_HAS_SLANG
-   
    last_key = key;
    last_map = map;
 
    Slrn_Full_Screen_Update = 1;
-   
+
    if ((*key->f.s == '.')
        || !SLang_execute_function (key->f.s))
      SLang_load_string(key->f.s);
-#endif
 }
 
 /*}}}*/
@@ -1654,7 +1662,7 @@ void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
 void slrn_set_prefix_argument (int rep) /*{{{*/
 {
    static int repeat;
-   
+
    repeat = rep;
    Slrn_Prefix_Arg_Ptr = &repeat;
 }
@@ -1666,12 +1674,12 @@ void slrn_digit_arg (void) /*{{{*/
    char buf[20];
    unsigned char key;
    unsigned int i;
-   
+
    i = 0;
    buf[i++] = (char) SLang_Last_Key_Char;
-   
+
    SLang_Key_TimeOut_Flag = 1;
-   
+
    while (1)
      {
 	buf[i] = 0;
@@ -1686,12 +1694,12 @@ void slrn_digit_arg (void) /*{{{*/
 
    SLang_Key_TimeOut_Flag = 0;
    slrn_set_prefix_argument (atoi (buf));
-   
+
    SLang_ungetkey (key);
-   if ((Slrn_Current_Mode != NULL) 
+   if ((Slrn_Current_Mode != NULL)
        && (Slrn_Current_Mode->keymap != NULL))
      slrn_do_keymap_key (Slrn_Current_Mode->keymap);
-   
+
    Slrn_Prefix_Arg_Ptr = NULL;
 }
 
@@ -1708,18 +1716,22 @@ int main (int argc, char **argv) /*{{{*/
 {
    if (-1 == main_init_and_parse_args (argc, argv))
      return 1;
-   
+
    if (-1 == slrn_select_group_mode ())
      return 1;
 
    slrn_push_suspension (1);
 
-#if SLRN_HAS_SLANG
    (void) slrn_run_hooks (HOOK_GROUP_MODE_STARTUP, 0);
-#endif
 
    if (Slrn_Batch) return 1;
-   
+
+   if (SLang_get_error ())
+     {
+	SLang_restart (1);	       /* prints any queued messages */
+	slrn_exit_error (_("An error was encountered during initialization."));
+     }
+
    while (Slrn_Current_Mode != NULL)
      {
 	if (SLKeyBoard_Quit)
@@ -1727,10 +1739,10 @@ int main (int argc, char **argv) /*{{{*/
 	     SLKeyBoard_Quit = 0;
 	     slrn_error (_("Quit!"));
 	  }
-	
+
 	(void) slrn_handle_interrupts ();
 
-	if (SLang_Error || !SLang_input_pending(0))
+	if (SLang_get_error() || !SLang_input_pending(0))
 	  {
 	     slrn_update_screen ();
 	  }
