@@ -2,7 +2,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999, 2007-2012 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 1994, 1999, 2007-2014 John E. Davis <jed@jedsoft.org>
  Copyright (c) 2001-2006  Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
@@ -17,7 +17,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
- 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -583,7 +583,7 @@ static SSL *alloc_ssl (void)
 
 #if SLTCP_HAS_GNUTLS_SUPPORT
    gnutls_global_set_log_function(tls_log_func);
-   gnutls_global_set_log_level(4711);
+   gnutls_global_set_log_level(0);
 #endif
    ssl = SSL_new (This_SSL_Ctx);
    if (ssl != NULL)
