@@ -6,6 +6,7 @@
 #include <signal.h>
 #include <dvidef.h>
 #include <descrip.h>
+#include <ssdef.h>
 
 #include <stat.h>
 #include <types.h>
@@ -24,4 +25,8 @@ extern char *slrn_vms_get_uaf_fullname (void);
 extern char *slrn_vms_getlogin (void);
 extern int vms_send_mail(char *, char *, char *);
 
+#if __VMS_VER < 70000000
+extern int pclose(FILE *);
+extern FILE *popen(char *, char *);
+#endif
 
